package applet;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;

import javax.swing.BoxLayout;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

public class SimpleWebClient extends JApplet {
	private static final long serialVersionUID = 1L;

	JComboBox  cmbMethod  = null;
	JTextField txtReqUri  = null;
	JComboBox  cmbVersion = null;
	JButton    btnOk	  = null;
	JTextArea  txtResult  = null;
	 
	private final String[] HTTP_METHOD = {
		"OPTIONS",
		"GET",
		"HEAD",
		"POST",
		"PUT",
		"DELETE",
		"TRACE",
		"CONNECT"
	};
  
	private final String[] HTTP_VERSION = {
		"HTTP/1.0",
		"HTTP/1.1"
	};

	public SimpleWebClient() {
		super();
		initialize();
	}
	
	private void initialize() {
		Container container = this.getContentPane();
		this.setSize(600, 400);
	    container.add(getControlPane());
	}
	
	@Override
	public void init() {
		super.init();
		
		String uri = getParameter("uri");
		txtReqUri.setText("http://" + ((uri == null)?"":uri));
		
		int method = 0;
		try {
			method = Integer.parseInt(getParameter("method"));
		} catch (Exception e) {}
		cmbMethod.setSelectedIndex(method);
		
		int version = 0;
		try {
			version = Integer.parseInt(getParameter("version"));
		} catch (Exception e) {}
		cmbVersion.setSelectedIndex(version);	
		
	}

	private JPanel getControlPane() {
		JPanel panelMain   = new JPanel();
		JPanel panelCtrl   = new JPanel();
		JPanel panelResutl = new JPanel();
		
	    panelCtrl.setLayout(new BoxLayout(panelCtrl,BoxLayout.X_AXIS));
	
	    cmbMethod  = new JComboBox(HTTP_METHOD);
	    
	    
	    txtReqUri  = new JTextField("");
	    cmbVersion = new JComboBox(HTTP_VERSION);
	    btnOk = new JButton("OK");
	    btnOk.addActionListener(
	    	new ActionListener() {
				public void actionPerformed(ActionEvent event) {
					doRequest();
				}
	    	}
	    );
	    
	    panelCtrl.add(new JLabel(" http method : "));
	    panelCtrl.add(cmbMethod);
	    panelCtrl.add(new JLabel(" request-uri : "));
	    panelCtrl.add(txtReqUri);
	    panelCtrl.add(new JLabel(" http version : "));
	    panelCtrl.add(cmbVersion);
	    panelCtrl.add(new JLabel(" "));
	    panelCtrl.add(btnOk);
	    
	    txtResult = new JTextArea(20, 60);
	    panelResutl.add( new JScrollPane(txtResult));
	    
	    panelMain.setLayout(new BorderLayout());
	    panelMain.add(panelCtrl,   BorderLayout.NORTH);
	    panelMain.add(panelResutl, BorderLayout.CENTER);
	    
	    return panelMain;
	}
	
	private void doRequest() {
		HostInfo hostInfo = parseHostInf(txtReqUri.getText());
		Socket        sok = null;
		PrintWriter   out = null;
		BufferedReader in = null;
		try {
			sok = new Socket(hostInfo.hostName, hostInfo.port);
			out = new PrintWriter(sok.getOutputStream(), true);
			in  = new BufferedReader(new InputStreamReader(sok.getInputStream(), "Shift_JIS"));
			
			String requri = getRequextUri();
		
			out.println(requri + "\r\n");
			out.flush();
			
			StringBuffer responseData = new StringBuffer();
			String line = null;
			while ((line = in.readLine()) != null) {
				responseData.append(line);
				responseData.append("\n");
			}
			txtResult.setText(responseData.toString());
			
		} catch (Exception e) {
			
			txtResult.setText(e.getMessage());
		} finally {
			try {
				if (in  != null) in.close();
				if (out != null) out.close();
				if (sok != null) sok.close();
			} catch (Exception e) {}
		}
	}
	
	private String getRequextUri() {

		return cmbMethod.getSelectedItem() + " " + txtReqUri.getText() + " " + cmbVersion.getSelectedItem();
	}
	
	private HostInfo parseHostInf(String url) {
		String host = "";
		String port = "80";
		int spos;
		int epos;
		int ppos;
		
		spos = url.indexOf("://");
		if (spos > 0) {
			host = url.substring(spos + 3);
		}
		epos = host.indexOf("/");
		if (epos > 0) {
			host = host.substring(0, epos);
		}
		ppos = host.indexOf(":");
		if (ppos > 0) {
			port = host.substring(ppos + 1);
			host = host.substring(0,ppos);
		}

		HostInfo result = new HostInfo();
		result.hostName = host;
		try {
			result.port = Integer.parseInt(port);
		} catch (Exception e) {
			result.port = 80;
		}
		
		return result;
	}
	
	class HostInfo {
		public String hostName;
		public int port;
	}
}
